@@ -22,15 +22,20 @@ use clap::ArgEnum;
 use core::fmt;
 use std::collections::HashMap;
 use std::result;
-use std::string::ParseError;
 
 use crate::error::{BallistaError, Result};
 
 use datafusion::arrow::datatypes::DataType;
-use log::warn;
 
 pub const BALLISTA_DEFAULT_SHUFFLE_PARTITIONS: &str = "ballista.shuffle.partitions";
+pub const BALLISTA_DEFAULT_BATCH_SIZE: &str = "ballista.batch.size";
+pub const BALLISTA_REPARTITION_JOINS: &str = "ballista.repartition.joins";
+pub const BALLISTA_REPARTITION_AGGREGATIONS: &str = "ballista.repartition.aggregations";
+pub const BALLISTA_REPARTITION_WINDOWS: &str = "ballista.repartition.windows";
+pub const BALLISTA_PARQUET_PRUNING: &str = "ballista.parquet.pruning";
 pub const BALLISTA_WITH_INFORMATION_SCHEMA: &str = "ballista.with_information_schema";
+/// give a plugin files dir, and then the dynamic library files in this dir will be load when scheduler state init.
+pub const BALLISTA_PLUGIN_DIR: &str = "ballista.plugin_dir";
 
 pub type ParseResult<T> = result::Result<T, String>;
 
@@ -136,6 +141,9 @@ impl BallistaConfig {
                     .parse::<bool>()
                     .map_err(|e| format!("{:?}", e))?;
             }
+            DataType::Utf8 => {
+                val.to_string();
+            }
             _ => {
                 return Err(format!("not support data type: {}", data_type));
             }
@@ -150,9 +158,27 @@ impl BallistaConfig {
             ConfigEntry::new(BALLISTA_DEFAULT_SHUFFLE_PARTITIONS.to_string(),
                 "Sets the default number of partitions to create when repartitioning query stages".to_string(),
                 DataType::UInt16, Some("2".to_string())),
+            ConfigEntry::new(BALLISTA_DEFAULT_BATCH_SIZE.to_string(),
+                             "Sets the default batch size".to_string(),
+                             DataType::UInt16, Some("8192".to_string())),
+            ConfigEntry::new(BALLISTA_REPARTITION_JOINS.to_string(),
+                             "Configuration for repartition joins".to_string(),
+                             DataType::Boolean, Some("true".to_string())),
+            ConfigEntry::new(BALLISTA_REPARTITION_AGGREGATIONS.to_string(),
+                             "Configuration for repartition aggregations".to_string(),
+                             DataType::Boolean,Some("true".to_string())),
+            ConfigEntry::new(BALLISTA_REPARTITION_WINDOWS.to_string(),
+                             "Configuration for repartition windows".to_string(),
+                             DataType::Boolean,Some("true".to_string())),
+            ConfigEntry::new(BALLISTA_PARQUET_PRUNING.to_string(),
+                             "Configuration for parquet prune".to_string(),
+                             DataType::Boolean,Some("true".to_string())),
             ConfigEntry::new(BALLISTA_WITH_INFORMATION_SCHEMA.to_string(),
                 "Sets whether enable information_schema".to_string(),
                 DataType::Boolean,Some("false".to_string())),
+            ConfigEntry::new(BALLISTA_PLUGIN_DIR.to_string(),
+                             "Sets the plugin dir".to_string(),
+                             DataType::Utf8,Some("".to_string())),
         ];
         entries
             .iter()
@@ -168,6 +194,30 @@ impl BallistaConfig {
         self.get_usize_setting(BALLISTA_DEFAULT_SHUFFLE_PARTITIONS)
     }
 
+    pub fn default_plugin_dir(&self) -> String {
+        self.get_string_setting(BALLISTA_PLUGIN_DIR)
+    }
+
+    pub fn default_batch_size(&self) -> usize {
+        self.get_usize_setting(BALLISTA_DEFAULT_BATCH_SIZE)
+    }
+
+    pub fn repartition_joins(&self) -> bool {
+        self.get_bool_setting(BALLISTA_REPARTITION_JOINS)
+    }
+
+    pub fn repartition_aggregations(&self) -> bool {
+        self.get_bool_setting(BALLISTA_REPARTITION_AGGREGATIONS)
+    }
+
+    pub fn repartition_windows(&self) -> bool {
+        self.get_bool_setting(BALLISTA_REPARTITION_WINDOWS)
+    }
+
+    pub fn parquet_pruning(&self) -> bool {
+        self.get_bool_setting(BALLISTA_PARQUET_PRUNING)
+    }
+
     pub fn default_with_information_schema(&self) -> bool {
         self.get_bool_setting(BALLISTA_WITH_INFORMATION_SCHEMA)
     }
@@ -195,6 +245,17 @@ impl BallistaConfig {
             v.parse::<bool>().unwrap()
         }
     }
+    fn get_string_setting(&self, key: &str) -> String {
+        if let Some(v) = self.settings.get(key) {
+            // infallible because we validate all configs in the constructor
+            v.to_string()
+        } else {
+            let entries = Self::valid_entries();
+            // infallible because we validate all configs in the constructor
+            let v = entries.get(key).unwrap().default_value.as_ref().unwrap();
+            v.to_string()
+        }
+    }
 }
 
 // an enum used to configure the scheduler policy
@@ -228,6 +289,7 @@ mod tests {
         let config = BallistaConfig::new()?;
         assert_eq!(2, config.default_shuffle_partitions());
         assert!(!config.default_with_information_schema());
+        assert_eq!("", config.default_plugin_dir().as_str());
         Ok(())
     }
 
@@ -246,6 +308,7 @@ mod tests {
     fn custom_config_invalid() -> Result<()> {
         let config = BallistaConfig::builder()
             .set(BALLISTA_DEFAULT_SHUFFLE_PARTITIONS, "true")
+            .set(BALLISTA_PLUGIN_DIR, "test_dir")
             .build();
         assert!(config.is_err());
         assert_eq!("General(\"Failed to parse user-supplied value 'ballista.shuffle.partitions' for configuration setting 'true': ParseIntError { kind: InvalidDigit }\")", format!("{:?}", config.unwrap_err()));
@@ -255,7 +318,6 @@ mod tests {
             .build();
         assert!(config.is_err());
         assert_eq!("General(\"Failed to parse user-supplied value 'ballista.with_information_schema' for configuration setting '123': ParseBoolError\")", format!("{:?}", config.unwrap_err()));
-
         Ok(())
     }
 }
