@@ -15,35 +15,28 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use std::fmt::Formatter;
+use std::any::Any;
 use std::sync::Arc;
-use std::{any::Any, pin::Pin};
 
 use crate::client::BallistaClient;
 use crate::serde::scheduler::{PartitionLocation, PartitionStats};
 
-use crate::utils::WrappedStream;
-use async_trait::async_trait;
 use datafusion::arrow::datatypes::SchemaRef;
-use datafusion::arrow::error::Result as ArrowResult;
-use datafusion::arrow::record_batch::RecordBatch;
-use datafusion::execution::runtime_env::RuntimeEnv;
+
+use datafusion::error::{DataFusionError, Result};
 use datafusion::physical_plan::expressions::PhysicalSortExpr;
 use datafusion::physical_plan::metrics::{
     ExecutionPlanMetricsSet, MetricBuilder, MetricsSet,
 };
 use datafusion::physical_plan::{
-    DisplayFormatType, ExecutionPlan, Metric, Partitioning, SendableRecordBatchStream,
-    Statistics,
+    DisplayFormatType, ExecutionPlan, Partitioning, SendableRecordBatchStream, Statistics,
 };
-use datafusion::{
-    error::{DataFusionError, Result},
-    physical_plan::RecordBatchStream,
-};
-use futures::{future, Stream, StreamExt};
-use hashbrown::HashMap;
+use futures::{StreamExt, TryStreamExt};
+
+use datafusion::arrow::error::ArrowError;
+use datafusion::execution::context::TaskContext;
+use datafusion::physical_plan::stream::RecordBatchStreamAdapter;
 use log::info;
-use std::time::Instant;
 
 /// ShuffleReaderExec reads partitions that have already been materialized by a ShuffleWriterExec
 /// being executed by an executor
@@ -70,7 +63,6 @@ impl ShuffleReaderExec {
     }
 }
 
-#[async_trait]
 impl ExecutionPlan for ShuffleReaderExec {
     fn as_any(&self) -> &dyn Any {
         self
@@ -99,7 +91,7 @@ impl ExecutionPlan for ShuffleReaderExec {
     }
 
     fn with_new_children(
-        &self,
+        self: Arc<Self>,
         _children: Vec<Arc<dyn ExecutionPlan>>,
     ) -> Result<Arc<dyn ExecutionPlan>> {
         Err(DataFusionError::Plan(
@@ -107,27 +99,32 @@ impl ExecutionPlan for ShuffleReaderExec {
         ))
     }
 
-    async fn execute(
+    fn execute(
         &self,
         partition: usize,
-        _runtime: Arc<RuntimeEnv>,
+        _context: Arc<TaskContext>,
     ) -> Result<SendableRecordBatchStream> {
         info!("ShuffleReaderExec::execute({})", partition);
 
         let fetch_time =
             MetricBuilder::new(&self.metrics).subset_time("fetch_time", partition);
-        let timer = fetch_time.timer();
 
-        let partition_locations = &self.partition[partition];
-        let result = future::join_all(partition_locations.iter().map(fetch_partition))
-            .await
-            .into_iter()
-            .collect::<Result<Vec<_>>>()?;
-        timer.done();
+        let locations = self.partition[partition].clone();
+        let stream = locations.into_iter().map(move |p| {
+            let fetch_time = fetch_time.clone();
+            futures::stream::once(async move {
+                let timer = fetch_time.timer();
+                let r = fetch_partition(&p).await;
+                timer.done();
 
-        let result = WrappedStream::new(
-            Box::pin(futures::stream::iter(result).flatten()),
+                r.map_err(|e| ArrowError::ExternalError(Box::new(e)))
+            })
+            .try_flatten()
+        });
+
+        let result = RecordBatchStreamAdapter::new(
             Arc::new(self.schema.as_ref().clone()),
+            futures::stream::iter(stream).flatten(),
         );
         Ok(Box::pin(result))
     }
@@ -205,7 +202,7 @@ fn stats_for_partitions(
 
 async fn fetch_partition(
     location: &PartitionLocation,
-) -> Result<Pin<Box<dyn RecordBatchStream + Send + Sync>>> {
+) -> Result<SendableRecordBatchStream> {
     let metadata = &location.executor_meta;
     let partition_id = &location.partition_id;
     let mut ballista_client =
