@@ -16,7 +16,6 @@
 // under the License.
 
 #![doc = include_str!("../README.md")]
-#![allow(unused_imports)]
 pub const BALLISTA_VERSION: &str = env!("CARGO_PKG_VERSION");
 
 pub fn print_version() {
@@ -26,7 +25,10 @@ pub fn print_version() {
 pub mod client;
 pub mod config;
 pub mod error;
+pub mod event_loop;
 pub mod execution_plans;
+/// some plugins
+pub mod plugin;
 pub mod utils;
 
 #[macro_use]
