@@ -15,52 +15,60 @@
 // specific language governing permissions and limitations
 // under the License.
 
+use std::convert::TryInto;
+use std::sync::Arc;
+
+use prost::bytes::BufMut;
+use prost::Message;
+
+use datafusion::arrow::compute::SortOptions;
+use datafusion::arrow::datatypes::SchemaRef;
+use datafusion::datafusion_data_access::object_store::local::LocalFileSystem;
+use datafusion::datasource::listing::PartitionedFile;
+use datafusion::execution::runtime_env::RuntimeEnv;
+use datafusion::logical_plan::window_frames::WindowFrame;
+use datafusion::logical_plan::FunctionRegistry;
+use datafusion::physical_plan::aggregates::AggregateExec;
+use datafusion::physical_plan::aggregates::{create_aggregate_expr, AggregateMode};
+use datafusion::physical_plan::coalesce_batches::CoalesceBatchesExec;
+use datafusion::physical_plan::coalesce_partitions::CoalescePartitionsExec;
+use datafusion::physical_plan::cross_join::CrossJoinExec;
+use datafusion::physical_plan::empty::EmptyExec;
+use datafusion::physical_plan::explain::ExplainExec;
+use datafusion::physical_plan::expressions::{Column, PhysicalSortExpr};
+use datafusion::physical_plan::file_format::{
+    AvroExec, CsvExec, FileScanConfig, ParquetExec,
+};
+use datafusion::physical_plan::filter::FilterExec;
+use datafusion::physical_plan::hash_join::{HashJoinExec, PartitionMode};
+use datafusion::physical_plan::limit::{GlobalLimitExec, LocalLimitExec};
+use datafusion::physical_plan::projection::ProjectionExec;
+use datafusion::physical_plan::repartition::RepartitionExec;
+use datafusion::physical_plan::sorts::sort::SortExec;
+use datafusion::physical_plan::union::UnionExec;
+use datafusion::physical_plan::windows::{create_window_expr, WindowAggExec};
+use datafusion::physical_plan::{
+    AggregateExpr, ExecutionPlan, Partitioning, PhysicalExpr, WindowExpr,
+};
+use datafusion_proto::from_proto::parse_expr;
+
 use crate::error::BallistaError;
 use crate::execution_plans::{
     ShuffleReaderExec, ShuffleWriterExec, UnresolvedShuffleExec,
 };
-use crate::serde::physical_plan::from_proto::parse_protobuf_hash_partitioning;
+use crate::serde::physical_plan::from_proto::{
+    parse_physical_expr, parse_protobuf_hash_partitioning,
+};
 use crate::serde::protobuf::physical_expr_node::ExprType;
 use crate::serde::protobuf::physical_plan_node::PhysicalPlanType;
 use crate::serde::protobuf::repartition_exec_node::PartitionMethod;
-use crate::serde::protobuf::ShuffleReaderPartition;
 use crate::serde::protobuf::{PhysicalExtensionNode, PhysicalPlanNode};
 use crate::serde::scheduler::PartitionLocation;
 use crate::serde::{
     byte_to_string, proto_error, protobuf, str_to_byte, AsExecutionPlan,
     PhysicalExtensionCodec,
 };
-use crate::{convert_box_required, convert_required, into_physical_plan, into_required};
-use datafusion::arrow::compute::SortOptions;
-use datafusion::arrow::datatypes::SchemaRef;
-use datafusion::datasource::object_store::local::LocalFileSystem;
-use datafusion::datasource::PartitionedFile;
-use datafusion::logical_plan::window_frames::WindowFrame;
-use datafusion::physical_plan::aggregates::create_aggregate_expr;
-use datafusion::physical_plan::coalesce_batches::CoalesceBatchesExec;
-use datafusion::physical_plan::coalesce_partitions::CoalescePartitionsExec;
-use datafusion::physical_plan::cross_join::CrossJoinExec;
-use datafusion::physical_plan::empty::EmptyExec;
-use datafusion::physical_plan::expressions::{Column, PhysicalSortExpr};
-use datafusion::physical_plan::file_format::{
-    AvroExec, CsvExec, FileScanConfig, ParquetExec,
-};
-use datafusion::physical_plan::filter::FilterExec;
-use datafusion::physical_plan::hash_aggregate::{AggregateMode, HashAggregateExec};
-use datafusion::physical_plan::hash_join::{HashJoinExec, PartitionMode};
-use datafusion::physical_plan::limit::{GlobalLimitExec, LocalLimitExec};
-use datafusion::physical_plan::projection::ProjectionExec;
-use datafusion::physical_plan::repartition::RepartitionExec;
-use datafusion::physical_plan::sorts::sort::SortExec;
-use datafusion::physical_plan::windows::{create_window_expr, WindowAggExec};
-use datafusion::physical_plan::{
-    AggregateExpr, ExecutionPlan, Partitioning, PhysicalExpr, WindowExpr,
-};
-use datafusion::prelude::ExecutionContext;
-use prost::bytes::BufMut;
-use prost::Message;
-use std::convert::TryInto;
-use std::sync::Arc;
+use crate::{convert_required, into_physical_plan, into_required};
 
 pub mod from_proto;
 pub mod to_proto;
@@ -87,7 +95,8 @@ impl AsExecutionPlan for PhysicalPlanNode {
 
     fn try_into_physical_plan(
         &self,
-        ctx: &ExecutionContext,
+        registry: &dyn FunctionRegistry,
+        runtime: &RuntimeEnv,
         extension_codec: &dyn PhysicalExtensionCodec,
     ) -> Result<Arc<dyn ExecutionPlan>, BallistaError> {
         let plan = self.physical_plan_type.as_ref().ok_or_else(|| {
@@ -97,51 +106,77 @@ impl AsExecutionPlan for PhysicalPlanNode {
             ))
         })?;
         match plan {
+            PhysicalPlanType::Explain(explain) => Ok(Arc::new(ExplainExec::new(
+                Arc::new(explain.schema.as_ref().unwrap().try_into()?),
+                explain
+                    .stringified_plans
+                    .iter()
+                    .map(|plan| plan.into())
+                    .collect(),
+                explain.verbose,
+            ))),
             PhysicalPlanType::Projection(projection) => {
-                let input: Arc<dyn ExecutionPlan> =
-                    into_physical_plan!(projection.input, ctx, extension_codec)?;
+                let input: Arc<dyn ExecutionPlan> = into_physical_plan!(
+                    projection.input,
+                    registry,
+                    runtime,
+                    extension_codec
+                )?;
                 let exprs = projection
                     .expr
                     .iter()
                     .zip(projection.expr_name.iter())
-                    .map(|(expr, name)| Ok((expr.try_into()?, name.to_string())))
+                    .map(|(expr, name)| Ok((parse_physical_expr(expr,registry)?, name.to_string())))
                     .collect::<Result<Vec<(Arc<dyn PhysicalExpr>, String)>, BallistaError>>(
                     )?;
                 Ok(Arc::new(ProjectionExec::try_new(exprs, input)?))
             }
             PhysicalPlanType::Filter(filter) => {
-                let input: Arc<dyn ExecutionPlan> =
-                    into_physical_plan!(filter.input, ctx, extension_codec)?;
+                let input: Arc<dyn ExecutionPlan> = into_physical_plan!(
+                    filter.input,
+                    registry,
+                    runtime,
+                    extension_codec
+                )?;
                 let predicate = filter
                     .expr
                     .as_ref()
+                    .map(|expr| parse_physical_expr(expr, registry))
+                    .transpose()?
                     .ok_or_else(|| {
                         BallistaError::General(
                             "filter (FilterExecNode) in PhysicalPlanNode is missing."
                                 .to_owned(),
                         )
-                    })?
-                    .try_into()?;
+                    })?;
                 Ok(Arc::new(FilterExec::try_new(predicate, input)?))
             }
             PhysicalPlanType::CsvScan(scan) => Ok(Arc::new(CsvExec::new(
-                decode_scan_config(scan.base_conf.as_ref().unwrap(), ctx)?,
+                decode_scan_config(scan.base_conf.as_ref().unwrap(), runtime)?,
                 scan.has_header,
                 str_to_byte(&scan.delimiter)?,
             ))),
             PhysicalPlanType::ParquetScan(scan) => {
+                let predicate = scan
+                    .pruning_predicate
+                    .as_ref()
+                    .map(|expr| parse_expr(expr, registry))
+                    .transpose()?;
                 Ok(Arc::new(ParquetExec::new(
-                    decode_scan_config(scan.base_conf.as_ref().unwrap(), ctx)?,
-                    // TODO predicate should be de-serialized
-                    None,
+                    decode_scan_config(scan.base_conf.as_ref().unwrap(), runtime)?,
+                    predicate,
                 )))
             }
             PhysicalPlanType::AvroScan(scan) => Ok(Arc::new(AvroExec::new(
-                decode_scan_config(scan.base_conf.as_ref().unwrap(), ctx)?,
+                decode_scan_config(scan.base_conf.as_ref().unwrap(), runtime)?,
             ))),
             PhysicalPlanType::CoalesceBatches(coalesce_batches) => {
-                let input: Arc<dyn ExecutionPlan> =
-                    into_physical_plan!(coalesce_batches.input, ctx, extension_codec)?;
+                let input: Arc<dyn ExecutionPlan> = into_physical_plan!(
+                    coalesce_batches.input,
+                    registry,
+                    runtime,
+                    extension_codec
+                )?;
                 Ok(Arc::new(CoalesceBatchesExec::new(
                     input,
                     coalesce_batches.target_batch_size as usize,
@@ -149,18 +184,22 @@ impl AsExecutionPlan for PhysicalPlanNode {
             }
             PhysicalPlanType::Merge(merge) => {
                 let input: Arc<dyn ExecutionPlan> =
-                    into_physical_plan!(merge.input, ctx, extension_codec)?;
+                    into_physical_plan!(merge.input, registry, runtime, extension_codec)?;
                 Ok(Arc::new(CoalescePartitionsExec::new(input)))
             }
             PhysicalPlanType::Repartition(repart) => {
-                let input: Arc<dyn ExecutionPlan> =
-                    into_physical_plan!(repart.input, ctx, extension_codec)?;
+                let input: Arc<dyn ExecutionPlan> = into_physical_plan!(
+                    repart.input,
+                    registry,
+                    runtime,
+                    extension_codec
+                )?;
                 match repart.partition_method {
                     Some(PartitionMethod::Hash(ref hash_part)) => {
                         let expr = hash_part
                             .hash_expr
                             .iter()
-                            .map(|e| e.try_into())
+                            .map(|e| parse_physical_expr(e, registry))
                             .collect::<Result<Vec<Arc<dyn PhysicalExpr>>, _>>()?;
 
                         Ok(Arc::new(RepartitionExec::try_new(
@@ -194,17 +233,21 @@ impl AsExecutionPlan for PhysicalPlanNode {
             }
             PhysicalPlanType::GlobalLimit(limit) => {
                 let input: Arc<dyn ExecutionPlan> =
-                    into_physical_plan!(limit.input, ctx, extension_codec)?;
+                    into_physical_plan!(limit.input, registry, runtime, extension_codec)?;
                 Ok(Arc::new(GlobalLimitExec::new(input, limit.limit as usize)))
             }
             PhysicalPlanType::LocalLimit(limit) => {
                 let input: Arc<dyn ExecutionPlan> =
-                    into_physical_plan!(limit.input, ctx, extension_codec)?;
+                    into_physical_plan!(limit.input, registry, runtime, extension_codec)?;
                 Ok(Arc::new(LocalLimitExec::new(input, limit.limit as usize)))
             }
             PhysicalPlanType::Window(window_agg) => {
-                let input: Arc<dyn ExecutionPlan> =
-                    into_physical_plan!(window_agg.input, ctx, extension_codec)?;
+                let input: Arc<dyn ExecutionPlan> = into_physical_plan!(
+                    window_agg.input,
+                    registry,
+                    runtime,
+                    extension_codec
+                )?;
                 let input_schema = window_agg
                     .input_schema
                     .as_ref()
@@ -227,15 +270,29 @@ impl AsExecutionPlan for PhysicalPlanNode {
                         })?;
 
                         match expr_type {
-                            ExprType::WindowExpr(window_node) => Ok(create_window_expr(
-                                &convert_required!(window_node.window_function)?,
-                                name.to_owned(),
-                                &[convert_box_required!(window_node.expr)?],
-                                &[],
-                                &[],
-                                Some(WindowFrame::default()),
-                                &physical_schema,
-                            )?),
+                            ExprType::WindowExpr(window_node) => {
+                                let window_node_expr = window_node
+                                    .expr
+                                    .as_ref()
+                                    .map(|e| parse_physical_expr(e.as_ref(), registry))
+                                    .transpose()?
+                                    .ok_or_else(|| {
+                                        proto_error(
+                                            "missing window_node expr expression"
+                                                .to_string(),
+                                        )
+                                    })?;
+
+                                Ok(create_window_expr(
+                                    &convert_required!(window_node.window_function)?,
+                                    name.to_owned(),
+                                    &[window_node_expr],
+                                    &[],
+                                    &[],
+                                    Some(WindowFrame::default()),
+                                    &physical_schema,
+                                )?)
+                            }
                             _ => Err(BallistaError::General(
                                 "Invalid expression for WindowAggrExec".to_string(),
                             )),
@@ -249,15 +306,21 @@ impl AsExecutionPlan for PhysicalPlanNode {
                     Arc::new((&input_schema).try_into()?),
                 )?))
             }
-            PhysicalPlanType::HashAggregate(hash_agg) => {
-                let input: Arc<dyn ExecutionPlan> =
-                    into_physical_plan!(hash_agg.input, ctx, extension_codec)?;
-                let mode = protobuf::AggregateMode::from_i32(hash_agg.mode).ok_or_else(|| {
-                    proto_error(format!(
-                        "Received a HashAggregateNode message with unknown AggregateMode {}",
+            PhysicalPlanType::Aggregate(hash_agg) => {
+                let input: Arc<dyn ExecutionPlan> = into_physical_plan!(
+                    hash_agg.input,
+                    registry,
+                    runtime,
+                    extension_codec
+                )?;
+                let mode = protobuf::AggregateMode::from_i32(hash_agg.mode).ok_or_else(
+                    || {
+                        proto_error(format!(
+                        "Received a AggregateNode message with unknown AggregateMode {}",
                         hash_agg.mode
                     ))
-                })?;
+                    },
+                )?;
                 let agg_mode: AggregateMode = match mode {
                     protobuf::AggregateMode::Partial => AggregateMode::Partial,
                     protobuf::AggregateMode::Final => AggregateMode::Final,
@@ -270,7 +333,8 @@ impl AsExecutionPlan for PhysicalPlanNode {
                     .iter()
                     .zip(hash_agg.group_expr_name.iter())
                     .map(|(expr, name)| {
-                        expr.try_into().map(|expr| (expr, name.to_string()))
+                        parse_physical_expr(expr, registry)
+                            .map(|expr| (expr, name.to_string()))
                     })
                     .collect::<Result<Vec<_>, _>>()?;
 
@@ -279,7 +343,7 @@ impl AsExecutionPlan for PhysicalPlanNode {
                     .as_ref()
                     .ok_or_else(|| {
                         BallistaError::General(
-                            "input_schema in HashAggregateNode is missing.".to_owned(),
+                            "input_schema in AggregateNode is missing.".to_owned(),
                         )
                     })?
                     .clone();
@@ -298,7 +362,7 @@ impl AsExecutionPlan for PhysicalPlanNode {
                         match expr_type {
                             ExprType::AggregateExpr(agg_node) => {
                                 let aggr_function =
-                                    protobuf::AggregateFunction::from_i32(
+                                    datafusion_proto::protobuf::AggregateFunction::from_i32(
                                         agg_node.aggr_function,
                                     )
                                         .ok_or_else(
@@ -310,23 +374,26 @@ impl AsExecutionPlan for PhysicalPlanNode {
                                             },
                                         )?;
 
+                                let input_phy_expr: Vec<Arc<dyn PhysicalExpr>> = agg_node.expr.iter()
+                                    .map(|e| parse_physical_expr(e, registry).unwrap()).collect();
+
                                 Ok(create_aggregate_expr(
                                     &aggr_function.into(),
                                     false,
-                                    &[convert_box_required!(agg_node.expr)?],
+                                    input_phy_expr.as_slice(),
                                     &physical_schema,
                                     name.to_string(),
                                 )?)
                             }
                             _ => Err(BallistaError::General(
-                                "Invalid aggregate  expression for HashAggregateExec"
+                                "Invalid aggregate expression for AggregateExec"
                                     .to_string(),
                             )),
                         }
                     })
                     .collect::<Result<Vec<_>, _>>()?;
 
-                Ok(Arc::new(HashAggregateExec::try_new(
+                Ok(Arc::new(AggregateExec::try_new(
                     agg_mode,
                     group,
                     physical_aggr_expr,
@@ -335,10 +402,18 @@ impl AsExecutionPlan for PhysicalPlanNode {
                 )?))
             }
             PhysicalPlanType::HashJoin(hashjoin) => {
-                let left: Arc<dyn ExecutionPlan> =
-                    into_physical_plan!(hashjoin.left, ctx, extension_codec)?;
-                let right: Arc<dyn ExecutionPlan> =
-                    into_physical_plan!(hashjoin.right, ctx, extension_codec)?;
+                let left: Arc<dyn ExecutionPlan> = into_physical_plan!(
+                    hashjoin.left,
+                    registry,
+                    runtime,
+                    extension_codec
+                )?;
+                let right: Arc<dyn ExecutionPlan> = into_physical_plan!(
+                    hashjoin.right,
+                    registry,
+                    runtime,
+                    extension_codec
+                )?;
                 let on: Vec<(Column, Column)> = hashjoin
                     .on
                     .iter()
@@ -377,19 +452,43 @@ impl AsExecutionPlan for PhysicalPlanNode {
                     &hashjoin.null_equals_null,
                 )?))
             }
+            PhysicalPlanType::Union(union) => {
+                let mut inputs: Vec<Arc<dyn ExecutionPlan>> = vec![];
+                for input in &union.inputs {
+                    inputs.push(input.try_into_physical_plan(
+                        registry,
+                        runtime,
+                        extension_codec,
+                    )?);
+                }
+                Ok(Arc::new(UnionExec::new(inputs)))
+            }
             PhysicalPlanType::CrossJoin(crossjoin) => {
-                let left: Arc<dyn ExecutionPlan> =
-                    into_physical_plan!(crossjoin.left, ctx, extension_codec)?;
-                let right: Arc<dyn ExecutionPlan> =
-                    into_physical_plan!(crossjoin.right, ctx, extension_codec)?;
+                let left: Arc<dyn ExecutionPlan> = into_physical_plan!(
+                    crossjoin.left,
+                    registry,
+                    runtime,
+                    extension_codec
+                )?;
+                let right: Arc<dyn ExecutionPlan> = into_physical_plan!(
+                    crossjoin.right,
+                    registry,
+                    runtime,
+                    extension_codec
+                )?;
                 Ok(Arc::new(CrossJoinExec::try_new(left, right)?))
             }
             PhysicalPlanType::ShuffleWriter(shuffle_writer) => {
-                let input: Arc<dyn ExecutionPlan> =
-                    into_physical_plan!(shuffle_writer.input, ctx, extension_codec)?;
+                let input: Arc<dyn ExecutionPlan> = into_physical_plan!(
+                    shuffle_writer.input,
+                    registry,
+                    runtime,
+                    extension_codec
+                )?;
 
                 let output_partitioning = parse_protobuf_hash_partitioning(
                     shuffle_writer.output_partitioning.as_ref(),
+                    registry,
                 )?;
 
                 Ok(Arc::new(ShuffleWriterExec::try_new(
@@ -422,7 +521,7 @@ impl AsExecutionPlan for PhysicalPlanNode {
             }
             PhysicalPlanType::Sort(sort) => {
                 let input: Arc<dyn ExecutionPlan> =
-                    into_physical_plan!(sort.input, ctx, extension_codec)?;
+                    into_physical_plan!(sort.input, registry, runtime, extension_codec)?;
                 let exprs = sort
                     .expr
                     .iter()
@@ -445,7 +544,7 @@ impl AsExecutionPlan for PhysicalPlanNode {
                                 })?
                                 .as_ref();
                             Ok(PhysicalSortExpr {
-                                expr: expr.try_into()?,
+                                expr: parse_physical_expr(expr,registry)?,
                                 options: SortOptions {
                                     descending: !sort_expr.asc,
                                     nulls_first: sort_expr.nulls_first,
@@ -476,11 +575,14 @@ impl AsExecutionPlan for PhysicalPlanNode {
                 let inputs: Vec<Arc<dyn ExecutionPlan>> = extension
                     .inputs
                     .iter()
-                    .map(|i| i.try_into_physical_plan(ctx, extension_codec))
+                    .map(|i| i.try_into_physical_plan(registry, runtime, extension_codec))
                     .collect::<Result<_, BallistaError>>()?;
 
-                let extension_node =
-                    extension_codec.try_decode(extension.node.as_slice(), &inputs)?;
+                let extension_node = extension_codec.try_decode(
+                    extension.node.as_slice(),
+                    &inputs,
+                    registry,
+                )?;
 
                 Ok(extension_node)
             }
@@ -497,7 +599,21 @@ impl AsExecutionPlan for PhysicalPlanNode {
         let plan_clone = plan.clone();
         let plan = plan.as_any();
 
-        if let Some(exec) = plan.downcast_ref::<ProjectionExec>() {
+        if let Some(exec) = plan.downcast_ref::<ExplainExec>() {
+            Ok(protobuf::PhysicalPlanNode {
+                physical_plan_type: Some(PhysicalPlanType::Explain(
+                    protobuf::ExplainExecNode {
+                        schema: Some(exec.schema().as_ref().into()),
+                        stringified_plans: exec
+                            .stringified_plans()
+                            .iter()
+                            .map(|plan| plan.into())
+                            .collect(),
+                        verbose: exec.verbose(),
+                    },
+                )),
+            })
+        } else if let Some(exec) = plan.downcast_ref::<ProjectionExec>() {
             let input = protobuf::PhysicalPlanNode::try_from_physical_plan(
                 exec.input().to_owned(),
                 extension_codec,
@@ -616,7 +732,7 @@ impl AsExecutionPlan for PhysicalPlanNode {
                     },
                 ))),
             })
-        } else if let Some(exec) = plan.downcast_ref::<HashAggregateExec>() {
+        } else if let Some(exec) = plan.downcast_ref::<AggregateExec>() {
             let groups = exec
                 .group_expr()
                 .iter()
@@ -654,8 +770,8 @@ impl AsExecutionPlan for PhysicalPlanNode {
                 extension_codec,
             )?;
             Ok(protobuf::PhysicalPlanNode {
-                physical_plan_type: Some(PhysicalPlanType::HashAggregate(Box::new(
-                    protobuf::HashAggregateExecNode {
+                physical_plan_type: Some(PhysicalPlanType::Aggregate(Box::new(
+                    protobuf::AggregateExecNode {
                         group_expr: groups,
                         group_expr_name: group_names,
                         aggr_expr: agg,
@@ -701,11 +817,15 @@ impl AsExecutionPlan for PhysicalPlanNode {
                 )),
             })
         } else if let Some(exec) = plan.downcast_ref::<ParquetExec>() {
+            let pruning_expr = exec
+                .pruning_predicate()
+                .map(|pred| pred.logical_expr().try_into())
+                .transpose()?;
             Ok(protobuf::PhysicalPlanNode {
                 physical_plan_type: Some(PhysicalPlanType::ParquetScan(
                     protobuf::ParquetScanExecNode {
                         base_conf: Some(exec.base_config().try_into()?),
-                        // TODO serialize predicates
+                        pruning_predicate: pruning_expr,
                     },
                 )),
             })
@@ -854,6 +974,19 @@ impl AsExecutionPlan for PhysicalPlanNode {
                     },
                 )),
             })
+        } else if let Some(union) = plan.downcast_ref::<UnionExec>() {
+            let mut inputs: Vec<PhysicalPlanNode> = vec![];
+            for input in union.inputs() {
+                inputs.push(protobuf::PhysicalPlanNode::try_from_physical_plan(
+                    input.to_owned(),
+                    extension_codec,
+                )?);
+            }
+            Ok(protobuf::PhysicalPlanNode {
+                physical_plan_type: Some(PhysicalPlanType::Union(
+                    protobuf::UnionExecNode { inputs },
+                )),
+            })
         } else {
             let mut buf: Vec<u8> = vec![];
             extension_codec.try_encode(plan_clone.clone(), &mut buf)?;
@@ -875,7 +1008,7 @@ impl AsExecutionPlan for PhysicalPlanNode {
 
 fn decode_scan_config(
     proto: &protobuf::FileScanExecConf,
-    ctx: &ExecutionContext,
+    runtime: &RuntimeEnv,
 ) -> Result<FileScanConfig, BallistaError> {
     let schema = Arc::new(convert_required!(proto.schema)?);
     let projection = proto
@@ -897,7 +1030,7 @@ fn decode_scan_config(
         .collect::<Result<Vec<_>, _>>()?;
 
     let object_store = if let Some(file) = file_groups.get(0).and_then(|h| h.get(0)) {
-        ctx.object_store(file.file_meta.path())?.0
+        runtime.object_store(file.file_meta.path())?.0
     } else {
         Arc::new(LocalFileSystem {})
     };
@@ -915,9 +1048,11 @@ fn decode_scan_config(
 
 #[macro_export]
 macro_rules! into_physical_plan {
-    ($PB:expr, $CTX:expr, $CODEC:expr) => {{
+    ($PB:expr, $REG:expr, $RUNTIME:expr, $CODEC:expr) => {{
         if let Some(field) = $PB.as_ref() {
-            field.as_ref().try_into_physical_plan(&$CTX, $CODEC)
+            field
+                .as_ref()
+                .try_into_physical_plan($REG, $RUNTIME, $CODEC)
         } else {
             Err(proto_error("Missing required field in protobuf"))
         }
@@ -926,38 +1061,51 @@ macro_rules! into_physical_plan {
 
 #[cfg(test)]
 mod roundtrip_tests {
-    use std::{convert::TryInto, sync::Arc};
+    use std::ops::Deref;
+    use std::sync::Arc;
 
-    use crate::serde::{AsExecutionPlan, BallistaCodec};
-    use datafusion::physical_plan::sorts::sort::SortExec;
-    use datafusion::prelude::ExecutionContext;
+    use datafusion::arrow::array::ArrayRef;
+    use datafusion::execution::context::ExecutionProps;
+    use datafusion::logical_expr::{BuiltinScalarFunction, Volatility};
+    use datafusion::logical_plan::create_udf;
+    use datafusion::physical_plan::functions;
+    use datafusion::physical_plan::functions::{
+        make_scalar_function, ScalarFunctionExpr,
+    };
+    use datafusion::physical_plan::projection::ProjectionExec;
     use datafusion::{
         arrow::{
             compute::kernels::sort::SortOptions,
             datatypes::{DataType, Field, Schema},
         },
+        datafusion_data_access::object_store::local::LocalFileSystem,
+        datasource::listing::PartitionedFile,
         logical_plan::{JoinType, Operator},
         physical_plan::{
+            aggregates::{AggregateExec, AggregateMode},
             empty::EmptyExec,
             expressions::{binary, col, lit, InListExpr, NotExpr},
             expressions::{Avg, Column, PhysicalSortExpr},
+            file_format::{FileScanConfig, ParquetExec},
             filter::FilterExec,
-            hash_aggregate::{AggregateMode, HashAggregateExec},
             hash_join::{HashJoinExec, PartitionMode},
             limit::{GlobalLimitExec, LocalLimitExec},
-            AggregateExpr, ColumnarValue, Distribution, ExecutionPlan, Partitioning,
-            PhysicalExpr,
+            sorts::sort::SortExec,
+            AggregateExpr, ExecutionPlan, Partitioning, PhysicalExpr, Statistics,
         },
+        prelude::SessionContext,
         scalar::ScalarValue,
     };
 
-    use super::super::super::error::Result;
-    use super::super::protobuf;
     use crate::execution_plans::ShuffleWriterExec;
     use crate::serde::protobuf::{LogicalPlanNode, PhysicalPlanNode};
+    use crate::serde::{AsExecutionPlan, BallistaCodec};
+
+    use super::super::super::error::Result;
+    use super::super::protobuf;
 
     fn roundtrip_test(exec_plan: Arc<dyn ExecutionPlan>) -> Result<()> {
-        let ctx = ExecutionContext::new();
+        let ctx = SessionContext::new();
         let codec: BallistaCodec<LogicalPlanNode, PhysicalPlanNode> =
             BallistaCodec::default();
         let proto: protobuf::PhysicalPlanNode =
@@ -966,8 +1114,40 @@ mod roundtrip_tests {
                 codec.physical_extension_codec(),
             )
             .expect("to proto");
+        let runtime = ctx.runtime_env();
         let result_exec_plan: Arc<dyn ExecutionPlan> = proto
-            .try_into_physical_plan(&ctx, codec.physical_extension_codec())
+            .try_into_physical_plan(
+                &ctx,
+                runtime.deref(),
+                codec.physical_extension_codec(),
+            )
+            .expect("from proto");
+        assert_eq!(
+            format!("{:?}", exec_plan),
+            format!("{:?}", result_exec_plan)
+        );
+        Ok(())
+    }
+
+    fn roundtrip_test_with_context(
+        exec_plan: Arc<dyn ExecutionPlan>,
+        ctx: SessionContext,
+    ) -> Result<()> {
+        let codec: BallistaCodec<LogicalPlanNode, PhysicalPlanNode> =
+            BallistaCodec::default();
+        let proto: protobuf::PhysicalPlanNode =
+            protobuf::PhysicalPlanNode::try_from_physical_plan(
+                exec_plan.clone(),
+                codec.physical_extension_codec(),
+            )
+            .expect("to proto");
+        let runtime = ctx.runtime_env();
+        let result_exec_plan: Arc<dyn ExecutionPlan> = proto
+            .try_into_physical_plan(
+                &ctx,
+                runtime.deref(),
+                codec.physical_extension_codec(),
+            )
             .expect("from proto");
         assert_eq!(
             format!("{:?}", exec_plan),
@@ -1034,7 +1214,7 @@ mod roundtrip_tests {
     }
 
     #[test]
-    fn rountrip_hash_aggregate() -> Result<()> {
+    fn rountrip_aggregate() -> Result<()> {
         let field_a = Field::new("a", DataType::Int64, false);
         let field_b = Field::new("b", DataType::Int64, false);
         let schema = Arc::new(Schema::new(vec![field_a, field_b]));
@@ -1048,7 +1228,7 @@ mod roundtrip_tests {
             DataType::Float64,
         ))];
 
-        roundtrip_test(Arc::new(HashAggregateExec::try_new(
+        roundtrip_test(Arc::new(AggregateExec::try_new(
             AggregateMode::Final,
             groups.clone(),
             aggregates.clone(),
@@ -1120,4 +1300,97 @@ mod roundtrip_tests {
             Some(Partitioning::Hash(vec![Arc::new(Column::new("a", 0))], 4)),
         )?))
     }
+
+    #[test]
+    fn roundtrip_parquet_exec_with_pruning_predicate() -> Result<()> {
+        let scan_config = FileScanConfig {
+            object_store: Arc::new(LocalFileSystem {}),
+            file_schema: Arc::new(Schema::new(vec![Field::new(
+                "col",
+                DataType::Utf8,
+                false,
+            )])),
+            file_groups: vec![vec![PartitionedFile::new(
+                "/path/to/file.parquet".to_string(),
+                1024,
+            )]],
+            statistics: Statistics {
+                num_rows: Some(100),
+                total_byte_size: Some(1024),
+                column_statistics: None,
+                is_exact: false,
+            },
+            projection: None,
+            limit: None,
+            table_partition_cols: vec![],
+        };
+
+        let predicate = datafusion::prelude::col("col").eq(datafusion::prelude::lit("1"));
+        roundtrip_test(Arc::new(ParquetExec::new(scan_config, Some(predicate))))
+    }
+
+    #[test]
+    fn roundtrip_builtin_scalar_function() -> Result<()> {
+        let field_a = Field::new("a", DataType::Int64, false);
+        let field_b = Field::new("b", DataType::Int64, false);
+        let schema = Arc::new(Schema::new(vec![field_a, field_b]));
+
+        let input = Arc::new(EmptyExec::new(false, schema.clone()));
+
+        let execution_props = ExecutionProps::new();
+
+        let fun_expr = functions::create_physical_fun(
+            &BuiltinScalarFunction::Abs,
+            &execution_props,
+        )?;
+
+        let expr = ScalarFunctionExpr::new(
+            "abs",
+            fun_expr,
+            vec![col("a", &schema)?],
+            &DataType::Int64,
+        );
+
+        let project =
+            ProjectionExec::try_new(vec![(Arc::new(expr), "a".to_string())], input)?;
+
+        roundtrip_test(Arc::new(project))
+    }
+
+    #[test]
+    fn roundtrip_scalar_udf() -> Result<()> {
+        let field_a = Field::new("a", DataType::Int64, false);
+        let field_b = Field::new("b", DataType::Int64, false);
+        let schema = Arc::new(Schema::new(vec![field_a, field_b]));
+
+        let input = Arc::new(EmptyExec::new(false, schema.clone()));
+
+        let fn_impl = |args: &[ArrayRef]| Ok(Arc::new(args[0].clone()) as ArrayRef);
+
+        let scalar_fn = make_scalar_function(fn_impl);
+
+        let udf = create_udf(
+            "dummy",
+            vec![DataType::Int64],
+            Arc::new(DataType::Int64),
+            Volatility::Immutable,
+            scalar_fn.clone(),
+        );
+
+        let expr = ScalarFunctionExpr::new(
+            "dummy",
+            scalar_fn,
+            vec![col("a", &schema)?],
+            &DataType::Int64,
+        );
+
+        let project =
+            ProjectionExec::try_new(vec![(Arc::new(expr), "a".to_string())], input)?;
+
+        let mut ctx = SessionContext::new();
+
+        ctx.register_udf(udf);
+
+        roundtrip_test_with_context(Arc::new(project), ctx)
+    }
 }
