@@ -15,6 +15,8 @@
 // specific language governing permissions and limitations
 // under the License.
 
+use std::collections::HashMap;
+use std::ops::Deref;
 use std::sync::atomic::{AtomicUsize, Ordering};
 use std::sync::mpsc::{Receiver, Sender, TryRecvError};
 use std::{sync::Arc, time::Duration};
@@ -23,7 +25,6 @@ use datafusion::physical_plan::ExecutionPlan;
 use log::{debug, error, info, warn};
 use tonic::transport::Channel;
 
-use ballista_core::serde::protobuf::ExecutorRegistration;
 use ballista_core::serde::protobuf::{
     scheduler_grpc_client::SchedulerGrpcClient, PollWorkParams, PollWorkResult,
     TaskDefinition, TaskStatus,
@@ -33,16 +34,24 @@ use crate::as_task_status;
 use crate::executor::Executor;
 use ballista_core::error::BallistaError;
 use ballista_core::serde::physical_plan::from_proto::parse_protobuf_hash_partitioning;
+use ballista_core::serde::scheduler::ExecutorSpecification;
 use ballista_core::serde::{AsExecutionPlan, AsLogicalPlan, BallistaCodec};
+use datafusion::execution::context::TaskContext;
 
 pub async fn poll_loop<T: 'static + AsLogicalPlan, U: 'static + AsExecutionPlan>(
     mut scheduler: SchedulerGrpcClient<Channel>,
     executor: Arc<Executor>,
-    executor_meta: ExecutorRegistration,
-    concurrent_tasks: usize,
     codec: BallistaCodec<T, U>,
 ) {
-    let available_tasks_slots = Arc::new(AtomicUsize::new(concurrent_tasks));
+    let executor_specification: ExecutorSpecification = executor
+        .metadata
+        .specification
+        .as_ref()
+        .unwrap()
+        .clone()
+        .into();
+    let available_tasks_slots =
+        Arc::new(AtomicUsize::new(executor_specification.task_slots as usize));
     let (task_status_sender, mut task_status_receiver) =
         std::sync::mpsc::channel::<TaskStatus>();
 
@@ -61,7 +70,7 @@ pub async fn poll_loop<T: 'static + AsLogicalPlan, U: 'static + AsExecutionPlan>
             tonic::Status,
         > = scheduler
             .poll_work(PollWorkParams {
-                metadata: Some(executor_meta.clone()),
+                metadata: Some(executor.metadata.clone()),
                 can_accept_task: available_tasks_slots.load(Ordering::SeqCst) > 0,
                 task_status,
             })
@@ -74,7 +83,6 @@ pub async fn poll_loop<T: 'static + AsLogicalPlan, U: 'static + AsExecutionPlan>
                 if let Some(task) = result.into_inner().task {
                     match run_received_tasks(
                         executor.clone(),
-                        executor_meta.id.clone(),
                         available_tasks_slots.clone(),
                         task_status_sender,
                         task,
@@ -106,7 +114,6 @@ pub async fn poll_loop<T: 'static + AsLogicalPlan, U: 'static + AsExecutionPlan>
 
 async fn run_received_tasks<T: 'static + AsLogicalPlan, U: 'static + AsExecutionPlan>(
     executor: Arc<Executor>,
-    executor_id: String,
     available_tasks_slots: Arc<AtomicUsize>,
     task_status_sender: Sender<TaskStatus>,
     task: TaskDefinition,
@@ -120,16 +127,44 @@ async fn run_received_tasks<T: 'static + AsLogicalPlan, U: 'static + AsExecution
     info!("Received task {}", task_id_log);
     available_tasks_slots.fetch_sub(1, Ordering::SeqCst);
 
+    let runtime = executor.runtime.clone();
+    let session_id = task.session_id;
+    let mut task_props = HashMap::new();
+    for kv_pair in task.props {
+        task_props.insert(kv_pair.key, kv_pair.value);
+    }
+
+    let mut task_scalar_functions = HashMap::new();
+    let mut task_aggregate_functions = HashMap::new();
+    // TODO combine the functions from Executor's functions and TaskDefintion's function resources
+    for scalar_func in executor.scalar_functions.clone() {
+        task_scalar_functions.insert(scalar_func.0.clone(), scalar_func.1);
+    }
+    for agg_func in executor.aggregate_functions.clone() {
+        task_aggregate_functions.insert(agg_func.0, agg_func.1);
+    }
+    let task_context = Arc::new(TaskContext::new(
+        task_id_log.clone(),
+        session_id,
+        task_props,
+        task_scalar_functions,
+        task_aggregate_functions,
+        runtime.clone(),
+    ));
+
     let plan: Arc<dyn ExecutionPlan> =
         U::try_decode(task.plan.as_slice()).and_then(|proto| {
             proto.try_into_physical_plan(
-                executor.ctx.as_ref(),
+                task_context.deref(),
+                runtime.deref(),
                 codec.physical_extension_codec(),
             )
         })?;
 
-    let shuffle_output_partitioning =
-        parse_protobuf_hash_partitioning(task.output_partitioning.as_ref())?;
+    let shuffle_output_partitioning = parse_protobuf_hash_partitioning(
+        task.output_partitioning.as_ref(),
+        task_context.as_ref(),
+    )?;
 
     tokio::spawn(async move {
         let execution_result = executor
@@ -138,6 +173,7 @@ async fn run_received_tasks<T: 'static + AsLogicalPlan, U: 'static + AsExecution
                 task_id.stage_id as usize,
                 task_id.partition_id as usize,
                 plan,
+                task_context,
                 shuffle_output_partitioning,
             )
             .await;
@@ -146,7 +182,7 @@ async fn run_received_tasks<T: 'static + AsLogicalPlan, U: 'static + AsExecution
         available_tasks_slots.fetch_add(1, Ordering::SeqCst);
         let _ = task_status_sender.send(as_task_status(
             execution_result,
-            executor_id,
+            executor.metadata.id.clone(),
             task_id,
         ));
     });
