@@ -30,7 +30,9 @@ use std::{
 };
 
 use ballista::context::BallistaContext;
-use ballista::prelude::{BallistaConfig, BALLISTA_DEFAULT_SHUFFLE_PARTITIONS};
+use ballista::prelude::{
+    BallistaConfig, BALLISTA_DEFAULT_BATCH_SIZE, BALLISTA_DEFAULT_SHUFFLE_PARTITIONS,
+};
 
 use datafusion::datasource::{MemTable, TableProvider};
 use datafusion::error::{DataFusionError, Result};
@@ -50,10 +52,8 @@ use datafusion::{
 };
 use datafusion::{
     arrow::util::pretty,
-    datasource::{
-        listing::{ListingOptions, ListingTable, ListingTableConfig},
-        object_store::local::LocalFileSystem,
-    },
+    datafusion_data_access::object_store::local::LocalFileSystem,
+    datasource::listing::{ListingOptions, ListingTable, ListingTableConfig},
 };
 
 use datafusion::datasource::file_format::csv::DEFAULT_CSV_EXTENSION;
@@ -83,9 +83,10 @@ struct BallistaBenchmarkOpt {
     #[structopt(short = "i", long = "iterations", default_value = "3")]
     iterations: usize,
 
-    // /// Batch size when reading CSV or Parquet files
-    // #[structopt(short = "s", long = "batch-size", default_value = "8192")]
-    // batch_size: usize,
+    /// Batch size when reading CSV or Parquet files
+    #[structopt(short = "s", long = "batch-size", default_value = "8192")]
+    batch_size: usize,
+
     /// Path to data files
     #[structopt(parse(from_os_str), required = true, short = "p", long = "path")]
     path: PathBuf,
@@ -174,6 +175,10 @@ struct BallistaLoadtestOpt {
     #[structopt(short = "n", long = "partitions", default_value = "2")]
     partitions: usize,
 
+    /// Batch size when reading CSV or Parquet files
+    #[structopt(short = "s", long = "batch-size", default_value = "8192")]
+    batch_size: usize,
+
     /// Path to data files
     #[structopt(parse(from_os_str), required = true, short = "p", long = "data-path")]
     path: PathBuf,
@@ -273,11 +278,10 @@ async fn main() -> Result<()> {
 async fn benchmark_datafusion(opt: DataFusionBenchmarkOpt) -> Result<Vec<RecordBatch>> {
     println!("Running benchmarks with the following options: {:?}", opt);
     let mut benchmark_run = BenchmarkRun::new(opt.query);
-    let config = ExecutionConfig::new()
+    let config = SessionConfig::new()
         .with_target_partitions(opt.partitions)
         .with_batch_size(opt.batch_size);
-    let mut ctx = ExecutionContext::with_config(config);
-    let runtime = ctx.state.lock().runtime_env.clone();
+    let ctx = SessionContext::with_config(config);
 
     // register tables
     for table in TABLES {
@@ -290,10 +294,9 @@ async fn benchmark_datafusion(opt: DataFusionBenchmarkOpt) -> Result<Vec<RecordB
         if opt.mem_table {
             println!("Loading table '{}' into memory", table);
             let start = Instant::now();
-
+            let task_ctx = ctx.task_ctx();
             let memtable =
-                MemTable::load(table_provider, Some(opt.partitions), runtime.clone())
-                    .await?;
+                MemTable::load(table_provider, Some(opt.partitions), task_ctx).await?;
             println!(
                 "Loaded table '{}' into memory in {} ms",
                 table,
@@ -310,8 +313,10 @@ async fn benchmark_datafusion(opt: DataFusionBenchmarkOpt) -> Result<Vec<RecordB
     let mut result: Vec<RecordBatch> = Vec::with_capacity(1);
     for i in 0..opt.iterations {
         let start = Instant::now();
-        let plan = create_logical_plan(&mut ctx, opt.query)?;
-        result = execute_query(&mut ctx, &plan, opt.debug).await?;
+        let plans = create_logical_plans(&ctx, opt.query)?;
+        for plan in plans {
+            result = execute_query(&ctx, &plan, opt.debug).await?;
+        }
         let elapsed = start.elapsed().as_secs_f64() * 1000.0;
         millis.push(elapsed as f64);
         let row_count = result.iter().map(|b| b.num_rows()).sum();
@@ -341,11 +346,14 @@ async fn benchmark_ballista(opt: BallistaBenchmarkOpt) -> Result<()> {
             BALLISTA_DEFAULT_SHUFFLE_PARTITIONS,
             &format!("{}", opt.partitions),
         )
+        .set(BALLISTA_DEFAULT_BATCH_SIZE, &format!("{}", opt.batch_size))
         .build()
         .map_err(|e| DataFusionError::Execution(format!("{:?}", e)))?;
 
     let ctx =
-        BallistaContext::remote(opt.host.unwrap().as_str(), opt.port.unwrap(), &config);
+        BallistaContext::remote(opt.host.unwrap().as_str(), opt.port.unwrap(), &config)
+            .await
+            .map_err(|e| DataFusionError::Execution(format!("{:?}", e)))?;
 
     // register tables with Ballista context
     let path = opt.path.to_str().unwrap();
@@ -356,20 +364,26 @@ async fn benchmark_ballista(opt: BallistaBenchmarkOpt) -> Result<()> {
     let mut millis = vec![];
 
     // run benchmark
-    let sql = get_query_sql(opt.query)?;
-    println!("Running benchmark with query {}:\n {}", opt.query, sql);
+    let queries = get_query_sql(opt.query)?;
+    println!(
+        "Running benchmark with queries {}:\n {:?}",
+        opt.query, queries
+    );
+    let mut batches = vec![];
     for i in 0..opt.iterations {
         let start = Instant::now();
-        let df = ctx
-            .sql(&sql)
-            .await
-            .map_err(|e| DataFusionError::Plan(format!("{:?}", e)))
-            .unwrap();
-        let batches = df
-            .collect()
-            .await
-            .map_err(|e| DataFusionError::Plan(format!("{:?}", e)))
-            .unwrap();
+        for sql in &queries {
+            let df = ctx
+                .sql(sql)
+                .await
+                .map_err(|e| DataFusionError::Plan(format!("{:?}", e)))
+                .unwrap();
+            batches = df
+                .collect()
+                .await
+                .map_err(|e| DataFusionError::Plan(format!("{:?}", e)))
+                .unwrap();
+        }
         let elapsed = start.elapsed().as_secs_f64() * 1000.0;
         millis.push(elapsed as f64);
         let row_count = batches.iter().map(|b| b.num_rows()).sum();
@@ -421,6 +435,7 @@ async fn loadtest_ballista(opt: BallistaLoadtestOpt) -> Result<()> {
             BALLISTA_DEFAULT_SHUFFLE_PARTITIONS,
             &format!("{}", opt.partitions),
         )
+        .set(BALLISTA_DEFAULT_BATCH_SIZE, &format!("{}", opt.batch_size))
         .build()
         .map_err(|e| DataFusionError::Execution(format!("{:?}", e)))?;
 
@@ -429,11 +444,15 @@ async fn loadtest_ballista(opt: BallistaLoadtestOpt) -> Result<()> {
     let mut clients = vec![];
 
     for _num in 0..concurrency {
-        clients.push(BallistaContext::remote(
-            opt.host.clone().unwrap().as_str(),
-            opt.port.unwrap(),
-            &config,
-        ));
+        clients.push(
+            BallistaContext::remote(
+                opt.host.clone().unwrap().as_str(),
+                opt.port.unwrap(),
+                &config,
+            )
+            .await
+            .map_err(|e| DataFusionError::Execution(format!("{:?}", e)))?,
+        );
     }
 
     // register tables with Ballista context
@@ -548,7 +567,7 @@ async fn register_tables(path: &str, file_format: &str, ctx: &BallistaContext) {
             }
             "parquet" => {
                 let path = format!("{}/{}", path, table);
-                ctx.register_parquet(table, &path)
+                ctx.register_parquet(table, &path, ParquetReadOptions::default())
                     .await
                     .map_err(|e| DataFusionError::Plan(format!("{:?}", e)))
                     .unwrap();
@@ -560,7 +579,8 @@ async fn register_tables(path: &str, file_format: &str, ctx: &BallistaContext) {
     }
 }
 
-fn get_query_sql(query: usize) -> Result<String> {
+/// Get the SQL statements from the specified query file
+fn get_query_sql(query: usize) -> Result<Vec<String>> {
     if query > 0 && query < 23 {
         let possibilities = vec![
             format!("queries/q{}.sql", query),
@@ -569,7 +589,14 @@ fn get_query_sql(query: usize) -> Result<String> {
         let mut errors = vec![];
         for filename in possibilities {
             match fs::read_to_string(&filename) {
-                Ok(contents) => return Ok(contents),
+                Ok(contents) => {
+                    return Ok(contents
+                        .split(';')
+                        .map(|s| s.trim())
+                        .filter(|s| !s.is_empty())
+                        .map(|s| s.to_string())
+                        .collect())
+                }
                 Err(e) => errors.push(format!("{}: {}", filename, e)),
             };
         }
@@ -584,13 +611,16 @@ fn get_query_sql(query: usize) -> Result<String> {
     }
 }
 
-fn create_logical_plan(ctx: &mut ExecutionContext, query: usize) -> Result<LogicalPlan> {
+/// Create a logical plan for each query in the specified query file
+fn create_logical_plans(ctx: &SessionContext, query: usize) -> Result<Vec<LogicalPlan>> {
     let sql = get_query_sql(query)?;
-    ctx.create_logical_plan(&sql)
+    sql.iter()
+        .map(|sql| ctx.create_logical_plan(sql.as_str()))
+        .collect::<Result<Vec<_>>>()
 }
 
 async fn execute_query(
-    ctx: &mut ExecutionContext,
+    ctx: &SessionContext,
     plan: &LogicalPlan,
     debug: bool,
 ) -> Result<Vec<RecordBatch>> {
@@ -608,8 +638,8 @@ async fn execute_query(
             displayable(physical_plan.as_ref()).indent()
         );
     }
-    let runtime = ctx.state.lock().runtime_env.clone();
-    let result = collect(physical_plan.clone(), runtime).await?;
+    let task_ctx = ctx.task_ctx();
+    let result = collect(physical_plan.clone(), task_ctx).await?;
     if debug {
         println!(
             "=== Physical plan with metrics ===\n{}\n",
@@ -632,8 +662,8 @@ async fn convert_tbl(opt: ConvertOpt) -> Result<()> {
             .delimiter(b'|')
             .file_extension(".tbl");
 
-        let config = ExecutionConfig::new().with_batch_size(opt.batch_size);
-        let mut ctx = ExecutionContext::with_config(config);
+        let config = SessionConfig::new().with_batch_size(opt.batch_size);
+        let ctx = SessionContext::with_config(config);
 
         // build plan to read the TBL file
         let mut csv = ctx.read_csv(&input_path, options).await?;
@@ -644,7 +674,7 @@ async fn convert_tbl(opt: ConvertOpt) -> Result<()> {
         }
 
         // create the physical plan
-        let csv = csv.to_logical_plan();
+        let csv = csv.to_logical_plan()?;
         let csv = ctx.optimize(&csv)?;
         let csv = ctx.create_physical_plan(&csv).await?;
 
@@ -1013,11 +1043,23 @@ mod tests {
         run_query(1).await
     }
 
+    #[ignore] // https://github.com/apache/arrow-datafusion/issues/159
+    #[tokio::test]
+    async fn run_2() -> Result<()> {
+        run_query(2).await
+    }
+
     #[tokio::test]
     async fn run_q3() -> Result<()> {
         run_query(3).await
     }
 
+    #[ignore] // https://github.com/apache/arrow-datafusion/issues/160
+    #[tokio::test]
+    async fn run_q4() -> Result<()> {
+        run_query(4).await
+    }
+
     #[tokio::test]
     async fn run_q5() -> Result<()> {
         run_query(5).await
@@ -1048,6 +1090,12 @@ mod tests {
         run_query(10).await
     }
 
+    #[ignore] // https://github.com/apache/arrow-datafusion/issues/163
+    #[tokio::test]
+    async fn run_q11() -> Result<()> {
+        run_query(11).await
+    }
+
     #[tokio::test]
     async fn run_q12() -> Result<()> {
         run_query(12).await
@@ -1063,11 +1111,51 @@ mod tests {
         run_query(14).await
     }
 
+    #[ignore] // https://github.com/apache/arrow-datafusion/issues/166
+    #[tokio::test]
+    async fn run_q15() -> Result<()> {
+        run_query(15).await
+    }
+
+    #[tokio::test]
+    async fn run_q16() -> Result<()> {
+        run_query(16).await
+    }
+
+    #[ignore] // https://github.com/apache/arrow-datafusion/issues/168
+    #[tokio::test]
+    async fn run_q17() -> Result<()> {
+        run_query(17).await
+    }
+
+    #[tokio::test]
+    async fn run_q18() -> Result<()> {
+        run_query(18).await
+    }
+
     #[tokio::test]
     async fn run_q19() -> Result<()> {
         run_query(19).await
     }
 
+    #[ignore] // https://github.com/apache/arrow-datafusion/issues/171
+    #[tokio::test]
+    async fn run_q20() -> Result<()> {
+        run_query(20).await
+    }
+
+    #[ignore] // https://github.com/apache/arrow-datafusion/issues/172
+    #[tokio::test]
+    async fn run_q21() -> Result<()> {
+        run_query(21).await
+    }
+
+    #[ignore] // https://github.com/apache/arrow-datafusion/issues/175
+    #[tokio::test]
+    async fn run_q22() -> Result<()> {
+        run_query(22).await
+    }
+
     /// Specialised String representation
     fn col_str(column: &ArrayRef, row_index: usize) -> String {
         if column.is_null(row_index) {
@@ -1280,12 +1368,12 @@ mod tests {
     }
 
     async fn run_query(n: usize) -> Result<()> {
-        // Tests running query with empty tables, to see whether they run succesfully.
+        // Tests running query with empty tables, to see whether they run successfully.
 
-        let config = ExecutionConfig::new()
+        let config = SessionConfig::new()
             .with_target_partitions(1)
             .with_batch_size(10);
-        let mut ctx = ExecutionContext::with_config(config);
+        let ctx = SessionContext::with_config(config);
 
         for &table in TABLES {
             let schema = get_schema(table);
@@ -1296,8 +1384,10 @@ mod tests {
             ctx.register_table(table, Arc::new(provider))?;
         }
 
-        let plan = create_logical_plan(&mut ctx, n)?;
-        execute_query(&mut ctx, &plan, false).await?;
+        let plans = create_logical_plans(&ctx, n)?;
+        for plan in plans {
+            execute_query(&ctx, &plan, false).await?;
+        }
 
         Ok(())
     }
@@ -1307,7 +1397,7 @@ mod tests {
             // load expected answers from tpch-dbgen
             // read csv as all strings, trim and cast to expected type as the csv string
             // to value parser does not handle data with leading/trailing spaces
-            let mut ctx = ExecutionContext::new();
+            let ctx = SessionContext::new();
             let schema = string_schema(get_answer_schema(n));
             let options = CsvReadOptions::new()
                 .schema(&schema)
@@ -1377,12 +1467,13 @@ mod tests {
             protobuf, AsExecutionPlan, AsLogicalPlan, BallistaCodec,
         };
         use datafusion::physical_plan::ExecutionPlan;
+        use std::ops::Deref;
 
         async fn round_trip_query(n: usize) -> Result<()> {
-            let config = ExecutionConfig::new()
+            let config = SessionConfig::new()
                 .with_target_partitions(1)
                 .with_batch_size(10);
-            let mut ctx = ExecutionContext::with_config(config);
+            let ctx = SessionContext::with_config(config);
             let codec: BallistaCodec<
                 protobuf::LogicalPlanNode,
                 protobuf::PhysicalPlanNode,
@@ -1412,56 +1503,63 @@ mod tests {
             }
 
             // test logical plan round trip
-            let plan = create_logical_plan(&mut ctx, n)?;
-            let proto: protobuf::LogicalPlanNode =
-                protobuf::LogicalPlanNode::try_from_logical_plan(
-                    &plan,
-                    codec.logical_extension_codec(),
-                )
-                .unwrap();
-            let round_trip: LogicalPlan = (&proto)
-                .try_into_logical_plan(&ctx, codec.logical_extension_codec())
-                .unwrap();
-            assert_eq!(
-                format!("{:?}", plan),
-                format!("{:?}", round_trip),
-                "logical plan round trip failed"
-            );
-
-            // test optimized logical plan round trip
-            let plan = ctx.optimize(&plan)?;
-            let proto: protobuf::LogicalPlanNode =
-                protobuf::LogicalPlanNode::try_from_logical_plan(
-                    &plan,
-                    codec.logical_extension_codec(),
-                )
-                .unwrap();
-            let round_trip: LogicalPlan = (&proto)
-                .try_into_logical_plan(&ctx, codec.logical_extension_codec())
-                .unwrap();
-            assert_eq!(
-                format!("{:?}", plan),
-                format!("{:?}", round_trip),
-                "optimized logical plan round trip failed"
-            );
-
-            // test physical plan roundtrip
-            if env::var("TPCH_DATA").is_ok() {
-                let physical_plan = ctx.create_physical_plan(&plan).await?;
-                let proto: protobuf::PhysicalPlanNode =
-                    protobuf::PhysicalPlanNode::try_from_physical_plan(
-                        physical_plan.clone(),
-                        codec.physical_extension_codec(),
+            let plans = create_logical_plans(&ctx, n)?;
+            for plan in plans {
+                let proto: protobuf::LogicalPlanNode =
+                    protobuf::LogicalPlanNode::try_from_logical_plan(
+                        &plan,
+                        codec.logical_extension_codec(),
                     )
                     .unwrap();
-                let round_trip: Arc<dyn ExecutionPlan> = (&proto)
-                    .try_into_physical_plan(&ctx, codec.physical_extension_codec())
+                let round_trip: LogicalPlan = (&proto)
+                    .try_into_logical_plan(&ctx, codec.logical_extension_codec())
                     .unwrap();
                 assert_eq!(
-                    format!("{:?}", physical_plan),
+                    format!("{:?}", plan),
                     format!("{:?}", round_trip),
-                    "physical plan round trip failed"
+                    "logical plan round trip failed"
                 );
+
+                // test optimized logical plan round trip
+                let plan = ctx.optimize(&plan)?;
+                let proto: protobuf::LogicalPlanNode =
+                    protobuf::LogicalPlanNode::try_from_logical_plan(
+                        &plan,
+                        codec.logical_extension_codec(),
+                    )
+                    .unwrap();
+                let round_trip: LogicalPlan = (&proto)
+                    .try_into_logical_plan(&ctx, codec.logical_extension_codec())
+                    .unwrap();
+                assert_eq!(
+                    format!("{:?}", plan),
+                    format!("{:?}", round_trip),
+                    "optimized logical plan round trip failed"
+                );
+
+                // test physical plan roundtrip
+                if env::var("TPCH_DATA").is_ok() {
+                    let physical_plan = ctx.create_physical_plan(&plan).await?;
+                    let proto: protobuf::PhysicalPlanNode =
+                        protobuf::PhysicalPlanNode::try_from_physical_plan(
+                            physical_plan.clone(),
+                            codec.physical_extension_codec(),
+                        )
+                        .unwrap();
+                    let runtime = ctx.runtime_env();
+                    let round_trip: Arc<dyn ExecutionPlan> = (&proto)
+                        .try_into_physical_plan(
+                            &ctx,
+                            runtime.deref(),
+                            codec.physical_extension_codec(),
+                        )
+                        .unwrap();
+                    assert_eq!(
+                        format!("{:?}", physical_plan),
+                        format!("{:?}", round_trip),
+                        "physical plan round trip failed"
+                    );
+                }
             }
 
             Ok(())
