@@ -18,23 +18,17 @@
 //! Execution functions
 
 use crate::{
-    command::Command,
+    command::{Command, OutputFormat},
     context::Context,
     helper::CliHelper,
-    print_format::{all_print_formats, PrintFormat},
     print_options::PrintOptions,
 };
-use datafusion::arrow::record_batch::RecordBatch;
-use datafusion::arrow::util::pretty;
-use datafusion::error::{DataFusionError, Result};
-use rustyline::config::Config;
+use datafusion::error::Result;
 use rustyline::error::ReadlineError;
 use rustyline::Editor;
 use std::fs::File;
 use std::io::prelude::*;
 use std::io::BufReader;
-use std::str::FromStr;
-use std::sync::Arc;
 use std::time::Instant;
 
 /// run and execute SQL statements and commands from a file, against a context with the given print options
@@ -78,21 +72,57 @@ pub async fn exec_from_lines(
     }
 }
 
+pub async fn exec_from_files(
+    files: Vec<String>,
+    ctx: &mut Context,
+    print_options: &PrintOptions,
+) {
+    let files = files
+        .into_iter()
+        .map(|file_path| File::open(file_path).unwrap())
+        .collect::<Vec<_>>();
+    for file in files {
+        let mut reader = BufReader::new(file);
+        exec_from_lines(ctx, &mut reader, print_options).await;
+    }
+}
+
 /// run and execute SQL statements and commands against a context with the given print options
-pub async fn exec_from_repl(ctx: &mut Context, print_options: &PrintOptions) {
+pub async fn exec_from_repl(ctx: &mut Context, print_options: &mut PrintOptions) {
     let mut rl = Editor::<CliHelper>::new();
     rl.set_helper(Some(CliHelper::default()));
     rl.load_history(".history").ok();
 
+    let mut print_options = print_options.clone();
+
     loop {
         match rl.readline("❯ ") {
             Ok(line) if line.starts_with('\\') => {
                 rl.add_history_entry(line.trim_end());
-                if let Ok(cmd) = &line[1..].parse::<Command>() {
+                let command = line.split_whitespace().collect::<Vec<_>>().join(" ");
+                if let Ok(cmd) = &command[1..].parse::<Command>() {
                     match cmd {
                         Command::Quit => break,
+                        Command::OutputFormat(subcommand) => {
+                            if let Some(subcommand) = subcommand {
+                                if let Ok(command) = subcommand.parse::<OutputFormat>() {
+                                    if let Err(e) =
+                                        command.execute(&mut print_options).await
+                                    {
+                                        eprintln!("{}", e)
+                                    }
+                                } else {
+                                    eprintln!(
+                                        "'\\{}' is not a valid command",
+                                        &line[1..]
+                                    );
+                                }
+                            } else {
+                                println!("Output format is {:?}.", print_options.format);
+                            }
+                        }
                         _ => {
-                            if let Err(e) = cmd.execute(ctx, print_options).await {
+                            if let Err(e) = cmd.execute(ctx, &mut print_options).await {
                                 eprintln!("{}", e)
                             }
                         }
@@ -103,7 +133,7 @@ pub async fn exec_from_repl(ctx: &mut Context, print_options: &PrintOptions) {
             }
             Ok(line) => {
                 rl.add_history_entry(line.trim_end());
-                match exec_and_print(ctx, print_options, line).await {
+                match exec_and_print(ctx, &print_options, line).await {
                     Ok(_) => {}
                     Err(err) => eprintln!("{:?}", err),
                 }
